#!/usr/bin/perl -w

# greygraph -- postfix mail traffic statistics
# copyright (c) 2000-2007 ETH Zurich
# copyright (c) 2000-2007 David Schweikert <david@schweikert.ch>
# modifed 2011 for grey Markus Neubauer <neubauer@std-service.com>
# released under the GNU General Public License

use RRDs;
use POSIX qw(uname);

my $VERSION = "1.14";

my $host = (POSIX::uname())[1];
my $scriptname = 'greygraph.cgi';
my $xpoints = 540;
my $points_per_sample = 3;
my $ypoints = 120;
my $ypoints_detail = 160;
my $rrd = '/var/lib/greygraph/greygraph.rrd'; # path to where the RRD database is
my $rrd_spam = '/var/lib/greygraph/greygraph_spam.rrd'; # path to where the Spam RRD database is
my $tmp_dir = '/var/cache/greygraph'; # temporary directory where to store the images

my @graphs = (
	{ title => 'Last Day',   seconds => 3600*24,        },
	{ title => 'Last Week',  seconds => 3600*24*7,      },
	{ title => 'Last Month', seconds => 3600*24*31,     },
	{ title => 'Last Year',  seconds => 3600*24*365, },
);

my %color = (
	whitelist => '999999', # rrggbb in hex
	new 	  => '009900',
	early     => 'AA0000', 
	spam      => '000000',
	awl       => 'DDBB00',
	reconnectok      => '000099',
);

sub rrd_graph(@)
{
	my ($range, $file, $ypoints, @rrdargs) = @_;
	my $step = $range*$points_per_sample/$xpoints;
	# choose carefully the end otherwise rrd will maybe pick the wrong RRA:
	my $end  = time; $end -= $end % $step;
	my $date = localtime(time);
	$date =~ s|:|\\:|g unless $RRDs::VERSION < 1.199908;

	my ($graphret,$xs,$ys) = RRDs::graph($file,
		'--imgformat', 'PNG',
		'--width', $xpoints,
		'--height', $ypoints,
		'--start', "-$range",
		'--end', $end,
		'--vertical-label', 'msgs/min',
		'--lower-limit', 0,
		'--units-exponent', 0, # don't show milli-messages/s
		'--lazy',
		'--color', 'SHADEA#ffffff',
		'--color', 'SHADEB#ffffff',
		'--color', 'BACK#ffffff',

		$RRDs::VERSION < 1.2002 ? () : ( '--slope-mode'),

		@rrdargs,

		'COMMENT:['.$date.']\r',
	);

	my $ERR=RRDs::error;
	die "ERROR: $ERR\n" if $ERR;
}

sub graph($$)
{
	my ($range, $file) = @_;
	my $step = $range*$points_per_sample/$xpoints;
	rrd_graph($range, $file, $ypoints,

		"DEF:new=$rrd:new:AVERAGE",
		"DEF:mnew=$rrd:new:MAX",
		"CDEF:rnew=new,60,*",
		"CDEF:rmnew=mnew,60,*",
		"CDEF:dnew=new,UN,0,new,IF,$step,*",
		"CDEF:snew=PREV,UN,dnew,PREV,IF,dnew,+",
		"LINE1:rnew#$color{new}:New          ",
		'GPRINT:snew:MAX:total\: %8.0lf msgs',
		'GPRINT:rnew:AVERAGE:avg\: %5.2lf msgs/min',
		'GPRINT:rmnew:MAX:max\: %4.0lf msgs/min\l',

		"DEF:reconnectok=$rrd_spam:reconnectok:AVERAGE",
		"DEF:mreconnectok=$rrd_spam:reconnectok:MAX",
		"CDEF:rreconnectok=reconnectok,60,*",
		"CDEF:dreconnectok=reconnectok,UN,0,reconnectok,IF,$step,*",
		"CDEF:sreconnectok=PREV,UN,dreconnectok,PREV,IF,dreconnectok,+",
		"CDEF:rmreconnectok=mreconnectok,60,*",
		"LINE1:rreconnectok#$color{reconnectok}:Reconnect ok ",
		'GPRINT:sreconnectok:MAX:total\: %8.0lf msgs',
		'GPRINT:rreconnectok:AVERAGE:avg\: %5.2lf msgs/min',
		'GPRINT:rmreconnectok:MAX:max\: %4.0lf msgs/min\l',

	);
}

sub graph_detail($$)
{
	my ($range, $file) = @_;
	my $step = $range*$points_per_sample/$xpoints;
	rrd_graph($range, $file, $ypoints_detail,

		"DEF:whitelist=$rrd:whitelist:AVERAGE",
		"DEF:mwhitelist=$rrd:whitelist:MAX",
		"CDEF:rwhitelist=whitelist,60,*",
		"CDEF:rmwhitelist=mwhitelist,60,*",
		"CDEF:dwhitelist=whitelist,UN,0,whitelist,IF,$step,*",
		"CDEF:swhitelist=PREV,UN,dwhitelist,PREV,IF,dwhitelist,+",
		"LINE1:rwhitelist#$color{whitelist}:Whitelist     ",
		'GPRINT:swhitelist:MAX:total\: %8.0lf msgs',
		'GPRINT:rwhitelist:AVERAGE:avg\: %5.2lf msgs/min',
		'GPRINT:rmwhitelist:MAX:max\: %4.0lf msgs/min\l',

		"DEF:awl=$rrd:awl:AVERAGE",
		"DEF:mawl=$rrd:awl:MAX",
		"CDEF:rawl=awl,60,*",
		"CDEF:dawl=awl,UN,0,awl,IF,$step,*",
		"CDEF:sawl=PREV,UN,dawl,PREV,IF,dawl,+",
		"CDEF:rmawl=mawl,60,*",
		"LINE1:rawl#$color{awl}:Auto whitelist",
		'GPRINT:sawl:MAX:total\: %8.0lf msgs',
		'GPRINT:rawl:AVERAGE:avg\: %5.2lf msgs/min',
		'GPRINT:rmawl:MAX:max\: %4.0lf msgs/min\l',

		"DEF:spam=$rrd_spam:spam:AVERAGE",
		"DEF:mspam=$rrd_spam:spam:MAX",
		"CDEF:rspam=spam,60,*",
		"CDEF:dspam=spam,UN,0,spam,IF,$step,*",
		"CDEF:sspam=PREV,UN,dspam,PREV,IF,dspam,+",
		"CDEF:rmspam=mspam,60,*",
		"LINE1:rspam#$color{spam}:Spam          ",
		'GPRINT:sspam:MAX:total\: %8.0lf msgs',
		'GPRINT:rspam:AVERAGE:avg\: %5.2lf msgs/min',
		'GPRINT:rmspam:MAX:max\: %4.0lf msgs/min\l',

		"DEF:early=$rrd:early:AVERAGE",
		"DEF:mearly=$rrd:early:MAX",
		"CDEF:rearly=early,60,*",
		"CDEF:dearly=early,UN,0,early,IF,$step,*",
		"CDEF:searly=PREV,UN,dearly,PREV,IF,dearly,+",
		"CDEF:rmearly=mearly,60,*",
		"STACK:rearly#$color{early}:Early connect ",
		'GPRINT:searly:MAX:total\: %8.0lf msgs',
		'GPRINT:rearly:AVERAGE:avg\: %5.2lf msgs/min',
		'GPRINT:rmearly:MAX:max\: %4.0lf msgs/min\l',

	);
}

sub print_html()
{
	print "Content-Type: text/html\n\n";

	print <<HEADER;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>Greylist Statistics for $host</title>
<meta http-equiv="Refresh" content="303; URL=/cgi-bin/mailgraph.cgi" />
<meta http-equiv="Pragma" content="no-cache" />
<META HTTP-EQUIV="Cache-Control" content="no-cache" />
<meta name="robots" content="noindex,nofollow" />
<link rel="stylesheet" href="/css/greygraph.css" type="text/css" />
</head>
<body>
HEADER

#        print '<ul id="topmenu"><li><a href="/cgi-bin/mailgraph.cgi">Mail Statistik</a></li> <li><a href="/cgi-bin/queuegraph.cgi">Mail Queue Statistik</a></li></ul>';

	print "<h1 id=\"GT\">Greylist Statistics for $host</h1>\n";

	print "<ul id=\"jump\"><small>\n";
	for my $n (0..$#graphs) {
		print "  <li><a href=\"#G$n\">$graphs[$n]{title}</a>&nbsp;</li>\n";
	}
#	print "</small></ul>\n";
	print "</small>&nbsp;\n";
        print '<li><a href="/cgi-bin/mailgraph.cgi"><strong>Mail Statistics</strong></a></li> <li><a href="/cgi-bin/queuegraph.cgi"><strong>Queue Statistics</strong></a></li></ul>';

	for my $n (0..$#graphs) {
		print "<div id=\"G$n\" style=\"background: #dddddd; width: 632px\">\n";
		print "<h2><a href=\"#GT\">TOP</a> - $graphs[$n]{title}</h2>\n";
		print "<p><img border=\"0\" src=\"$scriptname?${n}-n\" alt=\"greygraph $graphs[$n]{title} new\"/><br/>\n";
		print "<img border=\"0\" src=\"$scriptname?${n}-e\" alt=\"greygraph $graphs[$n]{title} detail\"/></p>\n";
		print "</div>\n";
	}

	print <<FOOTER;
<hr/>
<table><tr><td>
<a href="http://www.std-soft.com/bfaq/46-k-faq-server/117-greygraph-mail-statistik.html">Greygraph</a> $VERSION
by <a href="http://www.gichenbacher.de/kontakt">Markus Neubauer</a></td>
<td align="right">
<a href="http://oss.oetiker.ch/rrdtool/"><img src="http://www.std-soft.com/images/rrdtool.gif" alt="" width="120" height="34"/></a>
</td></tr></table>
</body></html>
FOOTER
}

sub send_image($)
{
	my ($file)= @_;

	-r $file or do {
		print "Content-type: text/plain\n\nERROR: can't find $file\n";
		exit 1;
	};

	print "Content-type: image/png\n";
	print "Content-length: ".((stat($file))[7])."\n";
	print "\n";
	open(IMG, $file) or die;
	my $data;
	print $data while read(IMG, $data, 16384)>0;
}

sub main()
{
	my $uri = $ENV{REQUEST_URI} || '';
	$uri =~ s/\/[^\/]+$//;
	$uri =~ s/\//,/g;
	$uri =~ s/(\~|\%7E)/tilde,/g;
	mkdir $tmp_dir, 0777 unless -d $tmp_dir;
	mkdir "$tmp_dir/$uri", 0777 unless -d "$tmp_dir/$uri";

	my $img = $ENV{QUERY_STRING};
	if(defined $img and $img =~ /\S/) {
		if($img =~ /^(\d+)-n$/) {
			my $file = "$tmp_dir/$uri/greygraph_$1.png";
			graph($graphs[$1]{seconds}, $file);
			send_image($file);
		}
		elsif($img =~ /^(\d+)-e$/) {
			my $file = "$tmp_dir/$uri/greygraph_$1_detail.png";
			graph_detail($graphs[$1]{seconds}, $file);
			send_image($file);
		}
		else {
			die "ERROR: invalid argument\n";
		}
	}
	else {
		print_html;
	}
}

main;
